<?php
/**
 * Created by IntelliJ IDEA.
 * User: hashlu
 * Date: 2/8/19
 * Time: 3:19 PM
 */

namespace Services\Lightspeed;


use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;

class LightspeedApiRateLimit
{
    const POST_UNIT = 10;
    const PUT_UNIT = 10;
    const DELETE_UNIT = 10;
    const GET_UNIT = 1;
    const MAX_RETRIES = 5;


    public function handler()
    {

        return function ($retries,
                         Request $request,
                         Response $response = null,
                         RequestException $exception = null) {

            if ($this->isRateLimitError($response)) {
                $this->checkWait($response, $this->unit($response));
                return true;
            }

            if ($retries >= self::MAX_RETRIES)
                return false;

            if ($response && $response->getStatusCode() == 401)
                return true;

            if ($this->isServerError($response))
                return true;

            $unit = $this->unit($request->getMethod());
            $this->checkWait($response, $unit);
            return false;
        };
    }


    private function unit($method)
    {
        if (is_string($method)) {
            $requestType = strtoupper($method);
            switch ($requestType) {
                case 'GET':
                    return self::GET_UNIT;
                case 'POST':
                    return self::POST_UNIT;
                case 'PUT':
                    return self::PUT_UNIT;
                case 'DELETE':
                    return self::DELETE_UNIT;
                default:
                    return 1;
            }
        }
        return -99;
    }

    /**
     * @param $response
     * @param $units
     */
    private function checkWait(Response $response, $units)
    {
        if ($units == -99) {
            // an error maybe?
            error_log('Possible error, sleeping 30');
            sleep(30);
        }
        if ($response == null)
            return;
        $headers = $response->getHeaders();

        if (!array_key_exists('X-LS-API-Drip-Rate', $headers) || !isset($headers['X-LS-API-Drip-Rate']))
            $drip = 1;
        else
            $drip = (float)$headers['X-LS-API-Drip-Rate'][0];

        if (array_key_exists('X-LS-API-Bucket-Level', $headers) && isset($headers['X-LS-API-Bucket-Level'])) {
            $level = explode('/', $headers['X-LS-API-Bucket-Level'][0]);
            $used = intval($level[0]);
            $size = intval($level[1]);
        } else {
            $used = 5;
            $size = 0;
        }

        if ($drip === 0) {
            $drip = 1;
        }

        $needed = $used + $units;
        if ($needed >= $size) {
            $wait = $needed - $size;
            $wait = $wait <= 0 ? 1 : $wait;
            error_log('Sleeping');
            sleep((int)$wait / $drip);
        }

    }


    function isServerError(Response $response = null)
    {
        return $response && $response->getStatusCode() >= 500;
    }

    function isRateLimitError(Response $response = null)
    {
        return $response && $response->getStatusCode() == 429;
    }

}

