<?php
/**
 * Created by IntelliJ IDEA.
 * User: hashlu
 * Date: 2/7/19
 * Time: 2:27 PM
 */

namespace Services\Lightspeed\api;


use GuzzleHttp\Exception\ClientException;

class InventoryApi extends RestClient
{

    public function __construct(LightspeedOauth $provider, $accountId)
    {
        parent::__construct($provider, $accountId);
    }

    public function getItem($itemId, $params = [], $asArray = true) {
        $response = $this->get('Item/' . $itemId, [
            'query'=> $params
        ]);

        return \GuzzleHttp\json_decode($response->getBody(), $asArray);
    }

    /**
     * @param $fields
     * @return mixed
     */
    public function createItem($fields)
    {
        $response =  $this->post('Item', [
            'json' => $fields
        ]);
        return json_decode($response->getBody());
    }

    /**
     * @param $itemId
     * @param $field
     * @return mixed
     */
    public function updateItem($itemId, $field) {
        $response = $this->put('Item/' . $itemId, [
           'json' => $field
        ]);
        return json_decode($response->getBody());
    }

    public function addFakeItemImage($fields, $fileName, $filePath) {
        $response = $this->post('Image', [
            'multipart' => [
                [
                    'name' => 'data',
                    'contents' => json_encode($fields)
                ],
                [
                    'name' => 'image',
                    'filename' => $fileName,
                    'contents' => fopen($filePath, 'r'),
                    'headers' => ['Content-Type' => 'image/jpeg']
                ]
            ]
        ]);
        return json_decode($response->getBody());
    }

    /**
     * @param array $fields
     * @return \Psr\Http\Message\ResponseInterface
     */
    public function createItemMatrix(array $fields)
    {
        return $this->post('ItemMatrix', $fields);
    }

    public function getItems($params = []) {
        $response = $this->get('Item', [
            'query' => $params
        ]);

        return json_decode($response->getBody());
    }

    public function getAllItems() {
        $itemList = [];
        $response = json_decode($this->get('Item')->getBody());

        if (!property_exists($response, 'Item'))
            return $itemList;
        $itemList = array_merge($itemList, $response->Item);
        $count = intval($response->{"@attributes"}->count);
        $offset = intval($response->{"@attributes"}->offset);
        while($offset <= $count) {
            $offset += 100;
            $response = $this->get('Item', [
                'query' =>[
                    'offset' => $offset,
                    'limit'=> 100,
                ]
            ]);
            $items = json_decode($response->getBody());
            if (property_exists($items, 'Item')) {
                $itemList = array_merge($itemList, $items->Item);
            }
        }
        return $itemList;
    }
    /**
     * @param array $fields
     * @return mixed
     */
    public function createCategory(array $fields)
    {
        $response = $this->post('Category', [
            'json' => $fields
        ]);
        return json_decode($response->getBody());
    }

    public function getCategories()
    {
        $categoryList = [];

        $response = json_decode($this->get('Category')->getBody(), true);
        if (!array_key_exists('Category', $response)) {
            return $categoryList;
        }
        $categoryList = array_merge($categoryList, $response['Category']);
        $count = intval($response['@attributes']['count']);
        $offset = intval($response['@attributes']['offset']);
        while ($offset <= $count) {
            $offset += 100;
            $response = $this->get('Category', ['query' => [
                'offset' => $offset,
                'limit' => 100
            ]]);
            $catResponse = json_decode($response->getBody(), true);

            if (array_key_exists('Category', $catResponse) && is_array($catResponse['Category'])) {
                $categoryList = array_merge($categoryList, $catResponse['Category']);
            }
        }
        return $categoryList;
    }

    public function deleteCategories(array $categories = [])
    {
        $categories = $this->getCategories();
        $tree = $this->buildCategoryTree($categories);
        foreach ($tree as $t) {
            if (array_key_exists('children', $t) && count($t['children']) > 0) {
                $this->traverseAndDeleteCategories($t['children']);
            }
            $this->deleteCategory($t['categoryID']);
        }
    }

    public function deleteCategory($categoryId)
    {
        try {
            error_log('Deleting ' . $categoryId);
            $resp = $this->delete('Category/' . $categoryId);
            return $resp;
        } catch (ClientException $e) {
            if ($e->getCode() != 400)
                throw $e;
            return null;
        }

    }

    public function buildCategoryTree($categoryList = [])
    {
        $categoryTrees = [];
        foreach ($categoryList as &$cat) {
            if ($cat['categoryID'] == '372') {
                error_log('id');
            }
            if ($cat['parentID'] == '0') { // at the root
                $children = [];
                foreach ($categoryList as $c) {
                    if ($c['leftNode'] > $cat['leftNode'] && $c['rightNode'] < $cat['rightNode']) {
                        $children[] = $c;
                    }
                }
                $cat['children'] = $children;
                $categoryTrees[] = $cat;
            }
        }

        foreach ($categoryTrees as &$t) {
            $t['children'] = $this->orderTree($t['children'], $t['categoryID']);
        }
        return $categoryTrees;
    }

    private function orderTree($elements = [], $parentID = '0')
    {
        $branch = [];
        foreach ($elements as $e) {
            if ($e['name'] == 'Tire') {
                error_log('t');
            }
            if ($e['categoryID'] == '372') {
                error_log('saaa');
            }
            if ($e['parentID'] == $parentID) {
                $children = $this->orderTree($elements, $e['categoryID']);
                if ($children) {
                    $e['children'] = $children;
                }
                $branch[] = $e;
            }
        }
        return $branch;
    }

    private function traverseAndDeleteCategories($elements)
    {
        if (!$elements) {
            return;
        }
        foreach ($elements as $c) {
            if (isset($c['children'])) {
                $this->traverseAndDeleteCategories($c['children']);
            }
            $this->deleteCategory($c['categoryID']);
        }
    }
}
