<?php
/**
 * Created by IntelliJ IDEA.
 * User: anger
 * Date: 5/23/16
 * Time: 6:21 PM
 */

namespace Services\Lightspeed\api;
use GuzzleHttp\Exception\GuzzleException;

define("LAST_SALE_SYNC_TIME", "/tmp/bbl_last_sale_sync");

/**
 * Class MOSAPI
 * @package mattanger\core
 */
class MOSAPI
{
    private $restClient;

    public function __construct(RestClient $restClient)
    {
        $this->restClient = $restClient;
    }


    /**
     * @param array $categories
     * @return array
     */
    public function getLightspeedBikes($categories = [])
    {
        $items = [];
        foreach ($categories as $category) {
            $count = 0;
            $data = json_decode($this->restClient->get("Item", [
                "categoryID" => $category["category_id"],
                "load_relations" => "all"
            ]));

            if ($data && !property_exists($data, "Item"))
                continue;
            if ($data->Item !== null && !is_array($data->Item)) { // we only have one?
                $items[] = $data->Item; // add the one
                continue; // move along here
            }
            $items = array_merge($items, $data->Item);
            if (property_exists($data, "@attributes")) {
                $count = $data->{"@attributes"}->count;
            }
            if ($count > 100) {
                $offset = 100;
                while ($offset < $count) {
                    $data = json_decode($this->restClient->get("Item", [
                        "categoryID" => $category["category_id"],
                        "load_relations" => 'all',
                        "offset" => $offset,
                        "limit" => 100
                    ]));
                    if (property_exists($data, "Item") && is_array($data->Item)) {
                        $items = array_merge($items, $data->Item);
                    } else {
                        $items[] = $data->Item;
                    }
                    $offset += 100;
                }
            }
        }
        return $items;
    }

    /**
     * Gets all of the sub categories for the Bikes in Lightspeed.
     *
     * @param $parentCategoryID
     * @return mixed
     */
    public function getLightpseedBikeCategories($parentCategoryID)
    {
        try {
            $r = $this->restClient->get("Category", [
                "parentID" => $parentCategoryID
            ]);
            $categories = json_decode($r);
            return $categories;
        } catch (GuzzleException $ge) {
        }
        return null;
    }

    public function getCategory($categoryId)
    {
        try {
            return json_decode($this->restClient->get("Category", [
                "categoryID" => $categoryId
            ]));
        } catch (GuzzleException $ge) {

        }
        return null;
    }

    /**
     * @param $shopID
     * @param array $params
     * @param string $load_relations
     * @return array
     */
    public function getWorkOrders($shopID, $params = [], $load_relations = 'all')
    {
        $params["shopID"] = $shopID;
        $params["load_relations"] = $load_relations;
        $workOrderArray = [];
        $workOrderObj = json_decode(
            $this->restClient->get("Workorder", $params)
        );

        if (property_exists($workOrderObj, "Workorder")) {
            $workOrderArray = array_merge($workOrderArray, $workOrderObj->Workorder);
//            print_r($workOrderObj->{"@attributes"});
        }

        if (property_exists($workOrderObj, "@attributes")) {
            $count = $workOrderObj->{"@attributes"}->count;
            $offset = 0;
            while ($offset < $count) {
                $offset += 100;
                $params["offset"] = $offset;
                $params["limit"] = 100;
                $workOrderObj = json_decode(
                    $this->restClient->get("Workorder", $params)
                );
                if ($workOrderObj && property_exists($workOrderObj, "Workorder")) {
                    $workOrderArray = array_merge($workOrderArray, $workOrderObj->Workorder);
                }
            }
        }

        return $workOrderArray;
    }


    /**
     * @return null
     */
    public function getWorkOrderStatus()
    {
        $woStatus = json_decode(
            $this->restClient->get("WorkorderStatus", [])
        );
        if (property_exists($woStatus, "WorkorderStatus")) {
            return $woStatus->WorkorderStatus;
        }
        return null;
    }


    /**
     * @param $shopID
     * @return array|null
     */
    private function getSaleLines($shopID)
    {
        $saleLines = json_decode($this->restClient->get("SaleLine", [
            "isWorkorder" => true,
            "shopID" => $shopID,
            "load_relations" => "all"
        ]));

        if (property_exists($saleLines, "SaleLine")) {
            $salelineMap = [];
            foreach ($saleLines->SaleLine as $sl) {
                $salelineMap[$sl->itemID . $sl->saleLineID] = $sl;
            }
            return $salelineMap;
        }

        return null;
    }

    /**
     * @param $saleLineId
     * @return mixed
     */
    public function getSaleLine($saleLineId)
    {
        $saleLine = json_decode($this->restClient->get("SaleLine", [
            "saleLineID" => $saleLineId,
            "load_relations" => 'all'
        ]));
        return $saleLine;
    }

    /**
     * @param null $lastModified
     * @return array
     */
    public function getSerials($lastModified = null)
    {
        $query = [
                "limit" => 100
            ];
        if ($lastModified) {
            $query['timeStamp'] = '>=' . $lastModified;
        }
        $serialNumbers = json_decode($this->restClient->get("Serialized", $query));

        //we're in trouble if we don't have a count
        if ($serialNumbers == null || !property_exists($serialNumbers, "@attributes"))
            return [];

        $count = $serialNumbers->{"@attributes"}->count;

        if ($count > 100) {
            $sn = $serialNumbers->Serialized;
            $offset = 0;
            while ($offset < $count) {
                $offset += 100;
                $serialNumbers = json_decode(
                    $this->restClient->get("Serialized", [
                        "offset" => $offset,
                        "limit" => 100
                    ])
                );
                if (property_exists($serialNumbers, "Serialized") && is_array($serialNumbers->Serialized))
                    $sn = array_merge($sn, $serialNumbers->Serialized);
            }

            return $sn;
        } else if (property_exists($serialNumbers, "Serialized")) {
            return $serialNumbers->Serialized;
        } else {
            return [];
        }
    }

    /**
     * @param $serialId
     * @return mixed
     */
    public function getSerial($serialId)
    {
        $sn = json_decode($this->restClient->get("Serialized", [
            "serializedID" => $serialId,
            "load_relations" => 'all'
        ]));
        return $sn;
    }

    /**
     * @param $saleLineId
     * @return mixed
     */
    public function getSerialBySaleLine($saleLineId)
    {
        $sn = json_decode($this->restClient->get('Serialized', [
            'saleLineID' => $saleLineId,
            'load_relations' => 'all'
        ]));
        return $sn;
    }

    /**
     * @return mixed
     */
    public function getEmployees()
    {
        $employees = json_decode($this->restClient->get("Employee"));
        return $employees;
    }

    /**
     * @param $workorderID
     * @return mixed
     */
    public function getWorkorder($workorderID)
    {
        $dat = $this->restClient->get("Workorder", [
            "workorderID" => $workorderID,
            "load_relations" => 'all'
        ]);
        $workorder = json_decode($dat);
        return $workorder;
    }

    /**
     * @param $customerID
     * @return mixed
     */
    public function getCustomer($customerID)
    {
        $customer = json_decode(
            $this->restClient->get('Customer', [
                "customerID" => $customerID,
                "load_relations" => "[\"Contact\"]"
            ])
        );
        if ($customer && property_exists($customer, 'Customer'))
            return $customer->Customer;
        return null;
    }

}
