<?php
/**
 * Created by PhpStorm.
 * User: anger
 * Date: 12/15/15
 * Time: 9:51 AM
 */

namespace Services\Lightspeed\api;
use GuzzleHttp;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Log;
use Psr\Http\Message\RequestInterface;
use Services\Lightspeed\LightspeedApiRateLimit;

class RestClient extends Client
{
    private static $nRetries = 3;

    private $account_id;

    private $response;

    private $provider;


    /**
     * RestClient constructor.
     * @param LightspeedOauth $provider
     * @param string $account_id
     */
    public function __construct(LightspeedOauth $provider, $account_id = '')
    {
//        $this->account_id = $accountId;
        $this->provider = $provider;
        $this->params = array();
        $handlerStack = GuzzleHttp\HandlerStack::create();
        $rateLimit = new LightspeedApiRateLimit();
        $handlerStack->push(GuzzleHttp\Middleware::retry($rateLimit->handler()));
        $handlerStack->push(GuzzleHttp\Middleware::mapRequest(function (RequestInterface $request) {
            return $request->withHeader('Authorization', $this->provider->getBearer());
        }));
        $this->account_id = trim($account_id);
        parent::__construct([
            'base_uri' => 'https://api.merchantos.com/API/Account/' . $this->account_id . '/',
            'handler' => $handlerStack,
            'headers' => [
                'Accept' => 'application/json'
            ]
        ]);
    }


    protected function url()
    {

    }

    public function getAccount()
    {
        $response = $this->get('/API/Account');
        return json_decode($response->getBody());
    }

    public function getShops() {
        $response = $this->get('Shop');
        return json_decode($response->getBody());
    }

    public function setAccountId($accountId) {
        $this->account_id = $accountId;
    }

    public function getAccountId() {
        return $this->account_id;
    }

    /**
     * @param \Psr\Http\Message\UriInterface|string $endpoint
     * @param array $id
     * @param $params
     * @return mixed|\Psr\Http\Message\ResponseInterface
     */
    public function getById($endpoint, $id, $params) {
        return $this->get($endpoint . '/' . $id, ['query' => $params]);
    }

    public function getIt($endpoint, $params) {
        $r = $this->get($endpoint, ['query' => $params]);
        return json_decode($r->getBody());
    }

    public function getAll($endpoint, $params = []) {
        $objlist = [];
        $response = $this->getIt($endpoint, array_merge(['load_relations' => 'all'], $params));
        if (!property_exists($response, $endpoint))
            return $objlist;
        $count = $this->getCount($response);
        $offset = $this->getOffset($response);
        while ($offset <= $count) {
            $offset += 100;
            $response = $this->getIt($endpoint, array_merge([
                'offset' => $offset,
                'limit' => 100,
                'load_relations' => 'all'
            ], $params));
            if (!property_exists($response, $endpoint)) continue;
            if (is_array($response->{$endpoint})) {
                $objlist = array_merge($objlist, $response->{$endpoint});
            } else {
                $objlist[] = $response->{$endpoint};
            }
        }
        return $objlist;
    }

    public function walkAll($endpoint,  \Closure $callback, $params = []) {
        $response = $this->getIt($endpoint, array_merge(['load_relations' => 'all'], $params));
        if (!property_exists($response, $endpoint)) {
            return;
        } else {
            $callback( is_array($response->{$endpoint}) ? $response->{$endpoint} : [$response->{$endpoint}]);
        }
        $count = $this->getCount($response);
        Log::info("importing " . $count . " altered workorders");
        $offset = $this->getOffset($response);
        while ($offset <= $count) {
            $offset += 100;
            $response = $this->getIt($endpoint, array_merge([
                'offset' => $offset,
                'limit' => 100,
                'load_relations' => 'all'
            ], $params));
            if (!property_exists($response, $endpoint)) continue;
            if (is_array($response->{$endpoint})) {
                $callback($response->{$endpoint});
            } else {
                $callback([$response->{$endpoint}]);
            }
        }
    }

    private function getOffset($response) {
        return intval($response->{"@attributes"}->{"offset"});
    }

    private function getCount($response) {
        return intval($response->{"@attributes"}->{"count"});
    }
}
