<?php
/**
 * Created by IntelliJ IDEA.
 * User: hashlu
 * Date: 3/15/19
 * Time: 8:20 AM
 */

namespace Services\Zoho\Builders;

use DOMDocument;

class CreatorBuilder
{
    private $doc;
    private $appList;
    private $root;
    private $applications;

    public function __construct()
    {
        $this->applications = [];
        $this->doc = new DOMDocument('1.0', 'UTF-8');
        $this->root = $this->doc->createElement('ZohoCreator');
        $this->doc->appendChild($this->root);
        $this->appList = $this->doc->createElement('applicationlist');
        $this->root->appendChild($this->appList);
    }

    public function addApplication($name)
    {
        $app = $this->doc->createElement('application');
        $formList = $this->doc->createElement('formlist');
        $app->setAttribute('name', $name);
        $app->appendChild($formList);
        $this->applications[$name] = $app;
        $this->appList->appendChild($app);
    }

    public function getDocument(): DOMDocument
    {
        return $this->doc;
    }

    /**
     * @param $appName
     * @param $formName
     * @param array $rows
     * @return \DOMElement
     */
    public function makeForm($appName, $formName)
    {
        $formList = $this->applications[$appName]->firstChild;
        $form = $this->doc->createElement('form');
        $form->setAttribute('name', $formName);

        $formList->appendChild($form);
        return $form;
    }

    public function formRow($row)
    {
        $add = $this->doc->createElement('add');
        foreach ($row as $name => $v) {
            $field = $this->doc->createElement('field');
            $field->setAttribute('name', $name);
            if ($v == "true") {
                $yesno = $this->yesNoOption(true);
                $field->appendChild($yesno);
            } else if ($v == "false") {
                $value = $this->yesNoOption(false);
                $field->appendChild($value);
            } else {
                $value = $this->doc->createElement('value');
                if (is_string($v))
                    $value->nodeValue = htmlspecialchars($v);
                else
                    $value->nodeValue = '???';
                $field->appendChild($value);
            }
            if ($v != null)
                $add->appendChild($field);
        }
        return $add;
    }

    public function createField($name) {
        $field = $this->getDocument()->createElement('field');
        $field->setAttribute("name", $name);
        return $field;
    }


    public function yesNoOption($val)
    {
        $options = $this->doc->createElement('options');
        $opt = $this->doc->createElement('option');
        $opt->nodeValue = $val ? htmlspecialchars("yes") : htmlspecialchars("no");
        $options->appendChild($opt);
        return $options;
    }

    private function checkRowSubform($row)
    {
        return array_key_exists('subform', $row);
    }

}


