<?php
/**
 * Created by IntelliJ IDEA.
 * User: hashlu
 * Date: 2/28/19
 * Time: 10:42 AM
 */

namespace Services\Zoho;

use GuzzleHttp\HandlerStack;
use SimpleXMLElement;

class ZohoApi extends \GuzzleHttp\Client
{
    private $accessToken;
    private $format;
    private $owner;
    private $appName;

    /**
     *
     * ZohoApi constructor.
     * @param string $accessToken
     * @param string $owner
     * @param array $config
     * @param string $format
     */
    public function __construct($accessToken = '', $owner = '', array $config = [], $format = 'json')
    {
        $this->accessToken = $accessToken;
        $this->format = $format;
        $this->owner = $owner;
        parent::__construct(array_merge([
            'base_uri' => 'https://creator.zoho.com/api/',
            'headers' => [],
            'handler' => $this->buildHandlers()
        ], $config));
    }


    /**
     * @param string $application
     * @param string $formName
     * @param array $data
     * @return mixed
     */
    public function editRecord($application = '', $formName = '', $data = [])
    {
        $body = array_merge($this->restFormParams(), $data);
        $response = $this->post($this->owner . '/json/' . $application . '/form/' . $formName . '/record/update', [
            'form_params' => $body
        ]);
        $d = rtrim(preg_replace('/^var[\s\S]+=\s/i', '', $response->getBody()), ';');
        return \GuzzleHttp\json_decode($d, true);
    }

    public function deleteRecord($application = '', $formName = '', $data = [])
    {
        $body = array_merge($this->restFormParams(), $data);
        $response = $this->post($this->owner . '/json/' . $application . '/form/' . $formName . '/record/delete', [
            'form_params' => $body
        ]);
        return json_decode($response->getBody(), true);
    }

    public function addRecord($application = '', $formName = '', $data = [])
    {
        $body = array_merge($this->restFormParams(), $data);
        $response = $this->post($this->owner . '/json/' . $application . '/form/' . $formName . '/record/add', [
            'form_params' => $body
        ]);
        return json_decode($response->getBody(), true);
    }

    public function viewRecord($application = '', $formName = '', $data = [])
    {
        $body = array_merge($this->restFormParams(), $data);
        $response = $this->post('json/' . $application . '/view/' . $formName, [
            'form_params' => $body
        ]);
         $d = rtrim(preg_replace('/^var[\s\S]+=\s/i', '', $response->getBody()), ';');
        return json_decode($d, true);
    }

    public function addManyRecords($xml)
    {
        $body = array_merge([
            'XMLString' => $xml
        ], $this->requireRPCParams());
        $response = $this->post('xml/write', [
            'form_params' => $body
        ]);
//        $doc = new \DOMDocument();
//        $doc->loadXML($response->getBody());
        return $response->getBody();
    }


    /**
     * Builds the handler stack for the client
     * put stuff like api rate limits here or stuff that needs to be done
     * before or after the request is made.
     * @return \GuzzleHttp\HandlerStack
     */
    private function buildHandlers()
    {
        $handlerStack = HandlerStack::create();
        return $handlerStack;
    }

    private function requireRPCParams()
    {
        return [
            'authtoken' => $this->accessToken,
            'scope' => 'creatorapi',
            'zc_ownername' => $this->owner,
        ];
    }

    private function restFormParams()
    {
        return [
            'authtoken' => $this->accessToken,
            'scope' => 'creatorapi',
            'raw' => 'true',
            'zc_ownername' => $this->owner
        ];
    }
}
