<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Session\Store;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Storage;
use Services\Lightspeed\api\LightspeedOauth;
use Services\Lightspeed\api\RestClient;

class TestLightspeedConnection extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'retail:test';

    private $provider;
    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Makes sure we can talk to Lightspeed';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }


    /**
     * @throws \Illuminate\Contracts\Filesystem\FileNotFoundException
     */
    public function handle()
    {
        $this->provider = App::make(LightspeedOauth::class);
        $this->provider->setAccessToken(json_decode(Storage::get('access_token.json'), true));
        $this->provider->setAccessTokenRefreshedCB(function($accessToken) {
            Storage::put('access_token.json', json_encode($accessToken));
            $this->info('Stored new access token');
        });
        $api = new RestClient($this->provider);
        $acct = $api->getAccount();
        $this->info($acct->Account->accountID);
        Storage::put('accountid', $acct->Account->accountID);
    }
}
