<?php


namespace App\Integrations\LsRetailToZoho;


use Services\Lightspeed\api\SaleApi;

class SalesTotals
{
    private $workorder;
    private $saleApi;
    private $salesTotals;

    /**
     * SalesTotals constructor.
     * @param SaleApi $saleApi
     * @param $workorder
     */
    public function __construct(SaleApi $saleApi, $workorder)
    {
        $this->saleApi = $saleApi;
        $this->workorder = $workorder;
        $this->salesTotals = [
            'invoiceSubTotal' => 0,
            'hst' => 0,
            'invoiceTotal' => 0,
            'laborTotal' => 0,
            'partsTotal' => 0,
            'miscItemTotal' => 0
        ];
    }

    /**
     * @return array
     */
    public function __invoke()
    {
        if ($this->workorder['saleID'] > 0) {
            $sale = $this->saleApi->getSale($this->workorder['saleID'], [
                'load_relations' => '["SaleLines","SaleLines.Item","SaleLines.TaxClass"]'
            ]);
            $sale = $sale['Sale'] ?? null;
            if ($sale != null) {
                $this->handleSale($sale);
                if (array_key_exists('SaleLines', $sale)) {
                    $saleLines = isAssoc($sale['SaleLines']['SaleLine']) ? [$sale['SaleLines']['SaleLine']] : $sale['SaleLines']['SaleLine'];
                    foreach ($saleLines as $sl) {
                        $this->handleTaxClassification($sl);
                    }
                }
            }
        } else {
            if (array_key_exists('WorkorderLines', $this->workorder)) {
                $workorderLines = isAssoc($this->workorder['WorkorderLines']['WorkorderLine'])
                    ? [$this->workorder['WorkorderLines']['WorkorderLine']]
                    : $workorderLines = $this->workorder['WorkorderLines']['WorkorderLine'];
                $this->handleWorkorderRows($workorderLines);
            }

            if (array_key_exists('WorkorderItems', $this->workorder)) {
                $workorderItems = isAssoc($this->workorder['WorkorderItems']['WorkorderItem'])
                    ? [$this->workorder['WorkorderItems']['WorkorderItem']] : $this->workorder['WorkorderItems']['WorkorderItem'];
                $this->handleWorkorderRows($workorderItems);
            }
        }

        return $this->formatOutput();
    }

    /**
     * @param $workorderRows
     */
    private function handleWorkorderRows($workorderRows)
    {

        foreach ($workorderRows as $workorderRow) {
            if ($workorderRow['saleLineID'] == 0) continue;
            $sl = $this->saleApi->getSaleLine($workorderRow['saleLineID'], [
                'load_relations' => '["TaxClass"]'
            ]);
            if (!array_key_exists('SaleLine', $sl)) continue;
            $sl = $sl['SaleLine'];
            $this->salesTotals['invoiceSubTotal'] += $sl['calcSubtotal'];
            $this->salesTotals['invoiceTotal'] += $sl['calcTotal'];
            $this->salesTotals['hst'] += $sl['calcTax1'];
            $this->handleTaxClassification($sl);
        }
    }

    /**
     * @param $sale
     */
    private function handleSale($sale)
    {
        $this->salesTotals['invoiceSubTotal'] = $sale['calcSubtotal'];
        $this->salesTotals['hst'] = $sale['taxTotal'];
        $this->salesTotals['invoiceTotal'] = $sale['calcTotal'];
    }

    /**
     * @param $sl
     */
    private function handleTaxClassification($sl)
    {
        if (array_key_exists('TaxClass', $sl)) {
            if ($sl['TaxClass']['name'] == 'Labor') {
                $this->salesTotals['laborTotal'] += doubleval($sl['calcSubtotal']);
            } else if ($sl['TaxClass']['name'] == 'Item') {
                $this->salesTotals['partsTotal'] += doubleval($sl['calcSubtotal']);
            } else {
                $this->salesTotals['miscItemTotal'] += doubleval($sl['calcSubtotal']);
            }
        }
    }

    /**
     * @return array
     */
    private function formatOutput()
    {
        return [
            'invoiceSubTotal' => sprintf("%.2f", (float)$this->salesTotals['invoiceSubTotal']),
            'hst' => sprintf("%.2f", (float)$this->salesTotals['hst']),
            'invoiceTotal' => sprintf("%.2f", (float)$this->salesTotals['invoiceTotal']),
            'partsTotal' => sprintf("%.2f", (float)$this->salesTotals['partsTotal']),
            'laborTotal' => sprintf("%.2f", (float)$this->salesTotals['laborTotal']),
        ];
    }
}
