<?php

namespace App\Integrations\LsRetailToZoho;

use Carbon\Carbon;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Storage;
use Services\Lightspeed\api\CustomerApi;
use Services\Lightspeed\api\EmployeeApi;
use Services\Lightspeed\api\InventoryApi;
use Services\Lightspeed\api\LightspeedOauth;
use Services\Lightspeed\api\SaleApi;
use Services\Zoho\Builders\CreatorBuilder;
use Services\Zoho\ZohoApi;

/**
 * Created by IntelliJ IDEA.
 * User: hashlu
 * Date: 3/17/19
 * Time: 9:46 AM
 */
class WorkorderDataSyncronizor
{
    private $provider;
    private $workorderApi;
    private $customerApi;
    private $inventoryApi;
    private $employeeApi;
    private $saleApi;
    private $statusMap;
    private $zohoApi;
    private $user;
    private $customerMap;
    private $employeeMap;

    /**
     * WorkorderDataSyncronizor constructor.
     */
    public function __construct($user, $zohoToken, $zohoOwner)
    {
        $this->user = $user;
        $this->statusMap = [];
        $this->employeeMap = [];
        $this->customerMap = [];
        $this->provider = App::make(LightspeedOauth::class);
        $this->provider->setAccessToken(\GuzzleHttp\json_decode(Storage::disk('user_data')
            ->get($user . '/retail_access_token.json'), true));
        $this->provider->setAccessTokenRefreshedCB(function ($accessToken) use ($user) {
            Storage::disk('user_data')->put($user . '/retail_access_token.json', \GuzzleHttp\json_encode($accessToken));
        });
        $this->workorderApi = new \Services\Lightspeed\api\WorkorderApi($this->provider, Storage::disk('user_data')->get($user . '/accountid'));
        $this->customerApi = new CustomerApi($this->provider, Storage::disk('user_data')->get($user . '/accountid'));
        $this->employeeApi = new EmployeeApi($this->provider, Storage::disk('user_data')->get($user . '/accountid'));
        $this->saleApi = new SaleApi($this->provider, Storage::disk('user_data')->get($user . '/accountid'));
        $this->inventoryApi = new InventoryApi($this->provider, Storage::disk('user_data')->get($user . '/accountid'));
        $this->zohoApi = new ZohoApi($zohoToken, $zohoOwner);
    }

    public function fullSync()
    {
        $woResp = $this->getWorkorders();
        $count = intval($woResp["@attributes"]['count']);
        $offset = intval($woResp["@attributes"]['offset']);
        // import for first 100 workorders
        $this->buildZohoImport($woResp['Workorder']);
        while ($offset <= $count) {
            $offset += 100;
            $woResp = $this->getWorkorders($offset);
            if (array_key_exists('Workorder', $woResp))
                $this->buildZohoImport($woResp['Workorder']);
        }

        Storage::disk('user_data')->put($this->user . '/last_sync', Carbon::now()->format(\DateTime::ISO8601));
    }

    public function updateEmployeeNames()
    {
        $zohoresponse = $this->zohoApi->viewRecord('workorder-manager', 'All_Workorder_Employees', [
            'startindex' => 0,
            'limit' => 5000,
        ]);

        foreach ($zohoresponse['Work_Orders'] as $z) {
            $r = $this->zohoApi->editRecord('workorder-manager', 'Work_Orders', [
                'criteria' => 'workorderID=' . $z['workorderID'],
                'employeeName' => $this->getEmployeeName($z['employeeID'])
            ]);
            usleep(.5 * 1000000);
        }

        $zohoresponse = $this->zohoApi->viewRecord('workorder-manager', 'All_Workorder_Lines');
        foreach ($zohoresponse['Workorder_Lines'] as $zl) {
            $this->zohoApi->editRecord('workorder-manager', 'Workorder_Lines', [
                'criteria' => 'workorderLineID=' . $zl['workorderLineID'],
                'emplyoeeName' => $this->getEmployeeName($zl['employeeID'])
            ]);
        }
        $zohoRequests = 1;
    }

    public function updateCustomSkus()
    {
        $startIdx = 0;
        $pgSize = 100;
        $zohoRequests = 0;
        $lastReq = time();
        $response = $this->zohoApi->viewRecord('workorder-manager', 'All_Workorder_Items');
        $zohoRequests++;
        error_log('does it break?');
        $itemMap = [];
        foreach ($response['Workorder_Items'] as $zItem) {
            if (!array_key_exists('ls-' . $zItem['workorderItemID'], $itemMap)) {
                $woItemResp = $this->workorderApi->getWorkorderItem($zItem['workorderItemID'], [], false);
                if (!property_exists($woItemResp, 'WorkorderItem')) {
                    $customSku = 'n/a';
                    $itemId = 0;
                } else {
                    if ($woItemResp->WorkorderItem->itemID == 0) {
                        $customSku = 'n/a';
                        $itemId = 0;
                    } else {
                        $itemResponse = $this->inventoryApi->getItem($woItemResp->WorkorderItem->itemID, [], false);
                        if (property_exists($itemResponse, 'Item')) {
                            $itemMap['ls-' . $zItem['workorderItemID']] = $itemResponse->Item;
                            $item = $itemResponse->Item;
                            $customSku = $item->customSku;
                            $itemId = $item->itemID;
                        } else {
                            $customSku = 'n/a';
                            $itemId = 0;
                        }
                    }
                }

            } else {
                $item = $itemMap['ls-' . $zItem['workorderItemID']];
                $customSku = $item->customSku;
                $itemId = $item->itemID;
            }
            $lastReq = time();
            $r = $this->zohoApi->editRecord('workorder-manager', 'Workorder_Items', [
                'criteria' => 'workorderItemID=' . $zItem['workorderItemID'],
                'customSku' => $customSku,
                'itemID' => $itemId,
            ]);
            usleep(.5 * 1000000);
            $zohoRequests++;

        }

    }

    public function incrementalSync()
    {
        $zohoRequestCount = 0;
        if (config('app.debug')){
            $time = Carbon::now('UTC')->subMinute(config('integrations.workorder_period'));
        } else {
            $time = Carbon::now('UTC')->subHour(24);
        }
        $woResp = $this->getWorkorders(0, 100, $time->format(\DateTime::ISO8601));

        $deleteList = [];
        $workorders = [];

        $count = intval($woResp["@attributes"]['count']);
        if ($count == 0) return;

        $offset = intval($woResp["@attributes"]['offset']);

        if (!array_key_exists('Workorder', $woResp)) return;

        if ($count > 100) {
            $workorders = $woResp['Workorder'];
            while ($offset <= $count) {
                $offset += 100;
                $woResp = $this->getWorkorders($offset, 100, $time->format(\DateTime::ISO8601));
                if (array_key_exists('Workorder', $woResp)) {
                    if (isAssoc($woResp['Workorder']))
                        $workorders = array_merge($workorders, [$woResp['Workorder']]);
                    else
                        $workorders = array_merge($workorders, $woResp['Workorder']);
                }
                Storage::disk('user_data')->append($this->user . '/last_wo_offset', $offset);
            }
        } else {
            if (isAssoc($woResp['Workorder']))
                $workorders[] = $woResp['Workorder'];
            else
                $workorders = $woResp['Workorder'];
        }


        foreach ($workorders as $workorder) {
            $deleteList[] = 'workorderID=' . $workorder['workorderID'];
        }

        if (!config('app.debug')) {
            $criteria = ['criteria' => implode(' || ', $deleteList)];
            $resp = $this->zohoApi->deleteRecord('workorder-manager', 'Work_Orders', $criteria);
            $zohoRequestCount++;
            $resp = $this->zohoApi->deleteRecord('workorder-manager', 'Workorder_Items', $criteria);
            $zohoRequestCount++;
            $resp = $this->zohoApi->deleteRecord('workorder-manager', 'Workorder_Lines', $criteria);
            $zohoRequestCount++;
        }
        $this->buildZohoImport($workorders);
        Storage::disk('user_data')->put($this->user . '/last_sync', Carbon::now('UTC')->format(\DateTime::ISO8601));
    }

    private function getWorkorders($offset = 0, $limit = 100, $lastMod = null)
    {
        $params = [
            'load_relations' => '["Customer","Employee","Serialized","WorkorderStatus","WorkorderLines","WorkorderItems","CustomFieldValues"]',
            'limit' => $limit,
            'offset' => $offset
        ];
        if ($lastMod)
            $params['timeStamp'] = ">=," . $lastMod;
        return $this->workorderApi->getWorkorders($params);
    }

    public function buildZohoImport($workorders = [])
    {
        $creatorBuilder = new CreatorBuilder();
        $creatorBuilder->addApplication('workorder-manager');
        $form = $creatorBuilder->makeForm("workorder-manager", 'Work_Orders');
        foreach ($workorders as $workorder) {
            if ($workorder['customerID'] > 0) {
                $customer = $this->customerApi->getCustomer($workorder['customerID'], [
                    'load_relations' => '["Contact"]'
                ], false);
                $customer = $customer->Customer ?? null;
            } else {
                $customer = null;
            }

            if (array_key_exists('Employee', $workorder))
                $this->storeEmplyoee($workorder['Employee']);

            $salesTotals = $this->getSalesTotals($workorder);

            $customerAddress = $this->getCustomerAddress($customer);
            $r = [
                'workorderID' => $workorder['workorderID'],
                'timeIn' => $this->parseDateTime($workorder['timeIn']),
                'etaOut' => $this->parseDateTime($workorder['etaOut']),
                'note' => $workorder['note'],
                'timeDone' => isset($workorder['timeDone']) ? $this->parseDateTime($workorder['timeDone']) : null,
                'warranty' => $workorder['warranty'], // todo: figure out bool
                'tax' => $workorder['tax'], // todo figure out bool,
                'hookIn' => $workorder['hookIn'],
                'hookOut' => $workorder['hookOut'],
                'saveParts' => $workorder['saveParts'],
                'customerID' => $workorder['customerID'],
                'employeeID' => $workorder['employeeID'],
                'serializedID' => $workorder['serializedID'],
                'status' => $this->getWorkorderStatus($workorder),
                'workorderStatusID' => $workorder['workorderStatusID'],
                'saleLineID' => $workorder['saleLineID'],
                'customerName' => $this->getCustomerName($customer),
                'customerEmail' => $this->getCustomerEmail($customer),
                'customerPhone' => $this->getCustomerPhone($customer),
                'serialNumber' => $this->getSerialNo($workorder),
                'serialItemDesc' => $this->getSerialItemDesc($workorder),
                'serialItemColor' => $this->getSerialItemColor($workorder),
                'lastModified' => $this->parseDateTime($workorder['timeStamp']),
                'employeeName' => $this->getEmployeeName($workorder['employeeID']),
                'customerFirstName' => $customer != null ? $customer->firstName : '',
                'customerLastName' => $customer != null ? $customer->lastName : '',
                'customerAddress' => $customerAddress['address'] ?? '',
                'customerCity' => $customerAddress['city'] ?? '',
                'customerState' => $customerAddress['state'] ?? '',
                'customerPostalCode' => $customerAddress['postalCode'] ? strval($customerAddress['postalCode']) : '',
                'customerHomePhone' => $this->getCustomerPhone($customer, "Home"),
                'customerMobilePhone' => $this->getCustomerPhone($customer, "Mobile"),
//                'deliveryDate' => '', // a custom field
                'invoiceSubTotal' => $salesTotals['invoiceSubTotal'],
                'hst' => $salesTotals['hst'],
                'invoiceTotal' => $salesTotals['invoiceTotal'],
                'partsTotal' => $salesTotals['partsTotal'],
                'laborTotal' => $salesTotals['laborTotal'],
            ];
            $add = $creatorBuilder->formRow($r);

            if (array_key_exists('WorkorderLines', $workorder)) {
                $subForm = $creatorBuilder->createField('workorder_lines');
                if (isAssoc($workorder['WorkorderLines']['WorkorderLine'])) {
                    $workorderLines = [$workorder['WorkorderLines']['WorkorderLine']];
                } else {
                    $workorderLines = $workorder['WorkorderLines']['WorkorderLine'];
                }
                foreach ($workorderLines as $woLine) {
                    $r = [
                        'workorderLineID' => $woLine['workorderLineID'],
                        'workorderID' => $woLine['workorderID'],
                        'note' => $woLine['note'],
                        'timeStamp' => $this->parseDateTime($woLine['timeStamp']),
                        'hours' => $woLine['hours'] ?? 0,
                        'minutes' => $woLine['minutes'] ?? 0,
                        'unitPriceOverride' => $woLine['unitPriceOverride'] ?? 0,
                        'unitQuantity' => $woLine['unityQuantity'] ?? 0,
                        'unitCost' => $woLine['unitCost'] ?? 0,
                        'done' => $woLine['done'],
                        'approved' => $woLine['approved'],
                        'warranty' => $woLine['warranty'],
                        'tax' => $woLine['tax'],
                        'salelineID' => $woLine['saleLineID'],
                        'employeeID' => $woLine['employeeID'],
                        'employeeName' => $this->getEmployeeName($woLine['employeeID']),
                        'itemID' => $woLine['itemID']
                    ];
                    $woLineRow = $creatorBuilder->formRow($r);
                    $subForm->appendChild($woLineRow);
                }
                $add->appendChild($subForm);
            }

            if (array_key_exists('WorkorderItems', $workorder)) {
                $itemsSubForm = $creatorBuilder->createField('workorder_items');
                if (isAssoc($workorder['WorkorderItems']['WorkorderItem'])) {
                    $workorderItems = [$workorder['WorkorderItems']['WorkorderItem']];
                } else {
                    $workorderItems = $workorder['WorkorderItems']['WorkorderItem'];
                }
                foreach ($workorderItems as $woItem) {
                    if (array_key_exists('itemID', $woItem)) {
                        $item = $this->inventoryApi->getItem($woItem['itemID'], [
                            'load_relations' => '["Images"]'
                        ], false);
                    } else {
                        $item = null;
                    }

                    if ($item && property_exists($item, 'Item')) {
                        $item = $item->Item;
                        $desc = $item->description;
                        $custSku = $item->customSku;
                    } else {
                        $desc = 'n/a';
                        $item = null;
                        $custSku = 'n/a';
                    }
                    $woItemRow = $creatorBuilder->formRow([
                        'workorderItemID' => $woItem['workorderItemID'],
                        'workorderID' => $woItem['workorderID'],
                        'description' => $desc,
                        'unitPrice' => $woItem['unitPrice'],
                        'unitQuantity' => $woItem['unitQuantity'],
                        'approved' => $woItem['approved'],
                        'warranty' => $woItem['warranty'],
                        'note' => $woItem['note'],
                        'image' => $this->getItemImage($item),
                        'customSku' => $custSku,
                        'itemID' => $woItem['itemID'],
                    ]);
                    $itemsSubForm->appendChild($woItemRow);
                }
                $add->appendChild($itemsSubForm);
                error_log('has items');
            }
            $form->appendChild($add);
        }

        if (!config('app.debug')) {
            $response = $this->zohoApi->addManyRecords($creatorBuilder->getDocument()->saveXML());
            file_put_contents('/tmp/test-response.xml', $response);
        }

    }

    private function getSalesTotals($workorder)
    {
        $salesTotals1 = new SalesTotals($this->saleApi, $workorder);
        return $salesTotals1();

        /*$salesTotals = [
            'invoiceSubTotal' => 0,
            'hst' => 0,
            'invoiceTotal' => 0,
            'laborTotal' => 0,
            'partsTotal' => 0,
            'miscItemTotal' => 0
        ];
        if ($workorder['saleID'] > 0) {
            $sale = $this->saleApi->getSale($workorder['saleID'], [
                'load_relations' => '["SaleLines","SaleLines.Item","SaleLines.TaxClass"]'
            ]);
            $sale = $sale['Sale'] ?? null;

            if ($sale != null) {
                $salesTotals['invoiceSubTotal'] = $sale['calcSubtotal'];
                $salesTotals['hst'] = $sale['taxTotal'];
                $salesTotals['invoiceTotal'] = $sale['calcTotal'];
                if (array_key_exists('SaleLines', $sale)) {
                    $saleLines = isAssoc($sale['SaleLines']['SaleLine']) ? [$sale['SaleLines']['SaleLine']] : $sale['SaleLines']['SaleLine'];
                    foreach ($saleLines as $sl) {
                        if (array_key_exists('TaxClass', $sl)) {
                            if ($sl['TaxClass']['name'] == 'Labor') {
                                $salesTotals['laborTotal'] += doubleval($sl['calcSubtotal']);
                            } else if ($sl['TaxClass']['name'] == 'Item') {
                                $salesTotals['partsTotal'] += doubleval($sl['calcSubtotal']);
                            } else {
                                $salesTotals['miscItemTotal'] += doubleval($sl['calcSubtotal']);
                            }
                        }
                    }
                }
                return $salesTotals;
            }
        } else {
            if (array_key_exists('WorkorderLines', $workorder)) {
                $workorderLines = isAssoc($workorder['WorkorderLines']['WorkorderLine'])
                    ? [$workorder['WorkorderLines']['WorkorderLine']]
                    : $workorderLines = $workorder['WorkorderLines']['WorkorderLine'];
                foreach ($workorderLines as $woLine) {
                    if ($woLine['saleLineID'] == 0) continue;
                    $sl = $this->saleApi->getSaleLine($woLine['saleLineID'], [
                        'load_relations' => '["TaxClass"]'
                    ]);
                    if (!array_key_exists('SaleLine', $sl)) continue;
                    $sl = $sl['SaleLine'];
                    $salesTotals['invoiceSubTotal'] += $sl['calcSubtotal'];
                    $salesTotals['invoiceTotal'] += $sl['calcTotal'];
                    $salesTotals['hst'] += $sl['calcTax1'];
                    if ($sl['TaxClass']['name'] == 'Labor') {
                        $salesTotals['laborTotal'] += doubleval($sl['calcSubtotal']);
                    } else if ($sl['TaxClass']['name'] == 'Item') {
                        $salesTotals['partsTotal'] += doubleval($sl['calcSubtotal']);
                    } else {
                        $salesTotals['miscItemTotal'] += doubleval($sl['calcSubtotal']);
                    }
                }
            }

            if (array_key_exists('WorkorderItems', $workorder)) {
                $workorderItems = isAssoc($workorder['WorkorderItems']['WorkorderItem'])
                    ? [$workorder['WorkorderItems']['WorkorderItem']] : $workorder['WorkorderItems']['WorkorderItem'];
                foreach ($workorderItems as $workorderItem) {
                    if ($workorderItem['saleLineID'] == 0) continue;
                    $sl = $this->saleApi->getSaleLine($workorderItem['saleLineID'], [
                        'load_relations' => '["TaxClass"]'
                    ]);
                    if (!array_key_exists('SaleLine', $sl)) continue;
                    $sl = $sl['SaleLine'];
                    $salesTotals['invoiceSubTotal'] += $sl['calcSubtotal'];
                    $salesTotals['invoiceTotal'] += $sl['calcTotal'];
                    $salesTotals['hst'] += $sl['calcTax1'];
                    if ($sl['TaxClass']['name'] == 'Labor') {
                        $salesTotals['laborTotal'] += doubleval($sl['calcSubtotal']);
                    } else if ($sl['TaxClass']['name'] == 'Item') {
                        $salesTotals['partsTotal'] += doubleval($sl['calcSubtotal']);
                    } else {
                        $salesTotals['miscItemTotal'] += doubleval($sl['calcSubtotal']);
                    }
                }
            }
        }
        return $salesTotals;*/
    }

    /**
     * @param $item
     * @return string
     */
    private function getItemImage($item)
    {
        if (!$item) return 'n/a';
        if (!property_exists($item, 'Images')) return 'n/a';
        if (is_array($item->Images->Image)) {
            return $item->Images->Image[0]->baseImageURL . $item->Images->Image[0]->publicID . '.jpg';
        } else {
            return $item->Images->Image->baseImageURL . $item->Images->Image->publicID . '.jpg';
        }
    }

    /**
     * @param $dt
     * @return string
     */
    private function parseDateTime($dt)
    {
        return Carbon::parse($dt)->format('d-M-Y h:i:s');
    }

    /**
     * @param $workorder
     * @return mixed|string
     */
    private function getWorkorderStatus($workorder)
    {
        if (!array_key_exists('WorkorderStatus', $workorder))
            return 'n/a';
        return $workorder['WorkorderStatus']['name'];
    }

    /**
     * @param $emplyoeeId
     * @return string
     */
    private function getEmployeeName($emplyoeeId)
    {
        $employee = $this->getEmployee($emplyoeeId);
        if ($employee) {
            return $employee['firstName'] . ' ' . $employee['lastName'];
        }
        return 'n/a';
    }

    /**
     *
     * @param $customer
     * @return string
     */
    private function getCustomerName($customer)
    {
        if (!$customer)
            return 'n/a';
        return sprintf('%s %s', $customer->firstName, $customer->lastName);
    }

    /**
     *
     * @param $customer
     * @return null
     */
    private function getCustomerEmail($customer)
    {
        if (!$customer || !property_exists($customer, 'Contact'))
            return null;

        $contact = $customer->Contact;
        if (empty($contact->Emails))
            return null;

        return is_array($contact->Emails->ContactEmail)
            ? $contact->Emails->ContactEmail[0]->address : $contact->Emails->ContactEmail->address;
    }

    /**
     * @param $customer
     * @param string $type
     * @return null
     */
    private function getCustomerPhone($customer, $type = "Mobile")
    {
        if (!$customer || !property_exists($customer, 'Contact'))
            return null;
        $contact = $customer->Contact;
        if (empty($contact->Phones))
            return null;

        if (is_array($contact->Phones->ContactPhone) && $type != null) {
            $number = '';
            foreach ($contact->Phones->ContactPhone as $p) {
                if ($p->useType == $type) {
                    $number = $p->number;
                    break;
                }
            }
            return $number;
        } else {
            return $contact->Phones->ContactPhone->number;
        }
    }

    /**
     * @param $customer
     * @return array
     */
    private function getCustomerAddress($customer)
    {
        if (!$customer || !property_exists($customer, 'Contact')) {
            return [
                'address' => '',
                'city' => '',
                'state' => '',
                'postalCode' => '',
            ];
        }

        if (is_array($customer->Contact->Addresses)) {
            $addr = $customer->Contact->Addresses[0];
        } else {
            $addr = $customer->Contact->Addresses->ContactAddress;
        }

        return [
            'address' => $addr->address1 . ' ' . $addr->address2,
            'city' => $addr->city,
            'state' => $addr->state,
            'postalCode' => $addr->zip,
        ];
    }

    /**
     * @param $workorder
     * @return mixed|string
     */
    private function getSerialNo($workorder)
    {
        if (!array_key_exists('Serialized', $workorder))
            return 'n/a';
        return $workorder['Serialized']['serial'];
    }

    /**
     * @param $workorder
     * @return mixed|string
     */
    private function getSerialItemDesc($workorder)
    {
        if (!array_key_exists('Serialized', $workorder))
            return 'n/a';
        return $workorder['Serialized']['description'];
    }

    private function getSerialItemColor($workorder)
    {
        if (!array_key_exists('Serialized', $workorder))
            return 'n/a';
        return $workorder['Serialized']['colorName'];
    }

    private function getEmployee($employeeId)
    {
        if (array_key_exists('e-' . $employeeId, $this->employeeMap))
            return $this->employeeMap['e-' . $employeeId];

        $eReps = $this->employeeApi->getEmployee($employeeId);
        if (array_key_exists('Employee', $eReps)) {
            $this->employeeMap['e-' . $employeeId] = $eReps['Employee'];
            return $eReps['Employee'];
        }
        return null;
    }

    private function storeEmplyoee($employee)
    {
        if (!array_key_exists('e-' . $employee['employeeID'], $this->employeeMap)) {
            $this->employeeMap['e-' . $employee['employeeID']] = $employee;
        }
    }
}
