<?php

namespace App\Models;

use App\Models\Traits\AttributeCreate;
use Illuminate\Database\Eloquent\Model;

/**
 * @property int $orderID
 * @property string $orderedDate
 * @property string $receivedDate
 * @property string $arrivalDate
 * @property string $refNum
 * @property int $shipInstructions
 * @property int $stockInstructions
 * @property float $shipCost
 * @property float $otherCost
 * @property boolean $complete
 * @property boolean $archived
 * @property float $discount
 * @property float $totalDiscount
 * @property int $totalQuantity
 * @property int $vendorID
 * @property int $noteID
 * @property int $shopID
 * @property string $created_at
 * @property string $updated_at
 */
class Order extends Model
{
    use AttributeCreate;
    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'orders';

    /**
     * The primary key for the model.
     *
     * @var string
     */
    protected $primaryKey = 'orderID';

    /**
     * Indicates if the IDs are auto-incrementing.
     *
     * @var bool
     */
    public $incrementing = false;

    /**
     * @var array
     */
    protected $fillable = ['orderID', 'orderedDate', 'receivedDate', 'arrivalDate', 'refNum', 'shipInstructions', 'stockInstructions', 'shipCost', 'otherCost', 'complete', 'archived', 'discount', 'totalDiscount', 'totalQuantity', 'vendorID', 'noteID', 'shopID', 'created_at', 'updated_at', 'createdByEmployeeID'];


    public static function boot()
    {
        parent::boot(); // TODO: Change the autogenerated stub
    }

    protected $casts = [
        'orderedDate' => 'datetime',
        'receivedDate' => 'datetime',
        'arrivalDate' => 'datetime',
        'shipInstructions' => 'integer',
        'stockInstructions' => 'integer',
        'complete' => 'boolean',
        'archived' => 'boolean',
    ];

    public function setShipInstructionsAttribute($value) {
        $this->attributes['shipInstructions'] = empty($value) ? null : intval($value);
    }

    public function setStockInstructionsAttribute($value) {
        $this->attributes['stockInstructions'] = empty($value) ? null : intval($value);
    }
}
