<?php

namespace App\Models;

use App\Models\Traits\AttributeCreate;
use App\Models\Traits\BoolStringConverter;
use Illuminate\Database\Eloquent\Model;

/**
 * @property int $workorderID
 * @property string $timeIn
 * @property string $timeOut
 * @property string $note
 * @property boolean $warranty
 * @property string $timeDone
 * @property boolean $tax
 * @property boolean $archived
 * @property string $hookIn
 * @property string $hookOut
 * @property boolean $saveParts
 * @property boolean $assignEmployeeToAll
 * @property int $customerID
 * @property int $discountID
 * @property int $employeeID
 * @property int $serializedID
 * @property int $shopID
 * @property int $saleID
 * @property int $saleLineID
 * @property int $workorderStatusID
 * @property string $created_at
 * @property string $updated_at
 */
class Workorder extends Model
{
    use AttributeCreate, BoolStringConverter;
    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'workorder';

    /**
     * The primary key for the model.
     *
     * @var string
     */
    protected $primaryKey = 'workorderID';

    /**
     * Indicates if the IDs are auto-incrementing.
     *
     * @var bool
     */
    public $incrementing = false;

    /**
     * @var array
     */
    protected $fillable = ['workorderID', 'timeIn', 'etaOut', 'note', 'warranty', 'timeDone', 'tax', 'archived', 'hookIn', 'hookOut', 'saveParts', 'assignEmployeeToAll', 'customerID', 'discountID', 'employeeID', 'serializedID', 'shopID', 'saleID', 'saleLineID', 'workorderStatusID', 'created_at', 'updated_at', 'internalNote', 'systemSku', 'timeStamp'];

    protected $casts = [
        'timeIn' => 'datetime',
        'etaOut' => 'datetime',
        'timeDone' => 'datetime',
        'timeStamp' => 'datetime',
    ];
    public function getWarrantyAttribute($value) {
        return $this->convertBoolString($value);
    }

    public function getTaxAttribute($value) {
        return $this->convertBoolString($value);
    }

    public function getSavePartsAttribute($value) {
        return $this->convertBoolString($value);
    }

    public function getAssignEmployeeToAllAttribute($value) {
        return $this->convertBoolString($value);
    }

    public function getArchivedAttribute($value) {
        return $this->convertBoolString($value);
    }

}
