<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateLSDbStructure extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {

        Schema::create('payment_type', function (Blueprint $table) {
            $table->integer('paymentTypeID');
            $table->text('name')->nullable(true);
            $table->boolean('requireCustomer')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->boolean('internalReserved')->nullable(true);
            $table->text('type')->nullable(true);
            $table->integer('refundAsPaymentTypeID')->nullable(true);

            $table->primary('paymentTypeID');
            $table->timestamps();
        });

        Schema::create('discount', function (Blueprint $table) {
            $table->integer('discountID');
            $table->text('name')->nullable(true);
            $table->float('discountAmount')->nullable(true);
            $table->float('discountPercent')->nullable(true);
            $table->boolean('requireCustomer')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('discountID');
            $table->timestamps();
        });


        Schema::create('tax_class', function(Blueprint $table) {
            $table->integer('taxClassID');
            $table->text('name')->nullable(true);
            $table->text('SaleLines')->nullable(true);
            $table->text('Items')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('taxClassID');
            $table->timestamps();
        });

        Schema::create('tax_category', function (Blueprint $table) {
            $table->integer('taxCategoryID');
            $table->boolean('isTaxInclusive')->nullable(true);
            $table->string('tax1Name')->nullable(true);
            $table->string('tax2Name')->nullable(true);
            $table->float('tax1Rate')->nullable(true);
            $table->float('tax2Rate')->nullable(true);

            $table->primary('taxCategoryID');
            $table->timestamps();
        });


        Schema::create('shop', function(Blueprint $table) {
            $table->integer('shopID');
            $table->text('name')->nullable(true);
            $table->float('serviceRate')->nullable(true);
            $table->string('timeZone')->nullable(true);
            $table->boolean('taxLabor')->nullable(true);
            $table->string('labelTitle')->nullable(true);
            $table->boolean('labelMsrp')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->integer('contactID')->nullable(true);
            $table->integer('taxCategoryID')->nullable(true);
            $table->integer('receiptSetupID')->nullable(true);
            $table->integer('ccGatewayID')->nullable(true);
            $table->integer('priceLevelID')->nullable(true);

            $table->primary('shopID');
            $table->timestamps();
        });


        Schema::create('vendor', function (Blueprint $table) {
            $table->integer('vendorID');
            $table->text('name')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->text('accountNumber')->nullable(true);
            $table->string('priceNumber')->nullable(true);
            $table->boolean('updatePrice')->nullable(true);
            $table->boolean('updateCost')->nullable(true);
            $table->boolean('updateDescription')->nullable(true);
            $table->boolean('shareSellThrough')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->integer('contactID')->nullable(true);

            $table->primary('vendorID');
            $table->timestamps();
        });

        Schema::create('special_order', function (Blueprint $table) {
            $table->integer('specialOrderID');
            $table->integer('unitQuantity')->nullable(true);
            $table->boolean('contacted')->nullable(true);
            $table->boolean('completed')->nullable(true);
            $table->integer('customerID')->nullable(true);
            $table->integer('shopID')->nullable(true);
            $table->integer('saleLineID')->nullable(true);
            $table->integer('orderLineID')->nullable(true);
            $table->integer('transferItemID')->nullable(true);

            $table->primary('specialOrderID');
            $table->timestamps();
        });

        Schema::create('tag', function (Blueprint $table) {
            $table->integer('tagID');
            $table->text('name')->nullable(true);

            $table->primary('tagID');
            $table->timestamps();
        });

        Schema::create('note', function (Blueprint $table) {
            $table->integer('objectID');
            $table->string('objectType');

            $table->text('note')->nullable(true);
            $table->boolean('isPublic')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary(['objectID', 'objectType']);
            $table->timestamps();
        });

        Schema::create('price_level', function (Blueprint $table) {
            $table->integer('priceLevelID');
            $table->text('name')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->boolean('canBeArchived')->nullable(true);
            $table->string('type')->nullable(true);

            $table->primary('priceLevelID');
            $table->timestamps();
        });

        Schema::create('catalog_vendor_item', function (Blueprint $table) {
            $table->integer('catalogVendorItemID');
            $table->string('vendorNumber')->nullable(true);
            $table->text('description')->nullable(true);
            $table->text('category')->nullable(true);
            $table->text('model')->nullable(true);
            $table->integer('retailUnit')->nullable(true);
            $table->string('unitOfMeasurement')->nullable(true);
            $table->float('cost')->nullable(true);
            $table->float('costLevel2')->nullable(true);
            $table->float('costLevel3')->nullable(true);
            $table->float('costLevel4')->nullable(true);
            $table->float('msrp')->nullable(true);
            $table->integer('breakQty')->nullable(true);
            $table->float('breakPrice')->nullable(true);
            $table->string('manufacturerNumber')->nullable(true);
            $table->text('brand')->nullable(true);
            $table->date('lastPriceChange')->nullable(true);
            $table->string('upc')->nullable(true);
            $table->string('ean')->nullable(true);
            $table->string('ean2')->nullable(true);
            $table->string('status')->nullable(true);
            $table->string('year')->nullable(true);
            $table->text('replacement')->nullable(true);
            $table->text('replacementDescription')->nullable(true);
            $table->integer('breakQty2')->nullable(true);
            $table->float('breakPrice2')->nullable(true);
            $table->integer('breakQty3')->nullable(true);
            $table->float('breakPrice3')->nullable(true);
            $table->string('colorName')->nullable(true);
            $table->string('sizeName')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->integer('lastQoh')->nullable(true);
            $table->integer('catalogVendorID')->nullable(true);

            $table->primary('catalogVendorItemID');
            $table->timestamps();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('l_s_db_structure');
    }
}
