<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateInventory extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        // create item & inventory DB
        Schema::create('item', function (Blueprint $table) {
            $table->integer('itemID');
            $table->bigInteger('systemSku')->nullable(true);
            $table->double('defaultCost')->nullable(true);
            $table->double('avgCost')->nullable(true);
            $table->boolean('tax')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->boolean('discountable')->nullable(true);
            $table->string('itemType')->nullable(true);
            $table->boolean('serialized')->nullable(true);
            $table->text('description')->nullable(true);
            $table->integer('modelYear')->nullable(true);
            $table->string('upc', 14)->nullable(true);
            $table->string('ean', 14)->nullable(true);
            $table->string('customSku')->nullable(true);
            $table->string('manufacturerSku')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->timestamp('createTime')->nullable(true);
            $table->boolean('publishToEcom')->nullable(true);
            $table->integer('categoryID')->nullable(true);
            $table->integer('taxClassID')->nullable(true);
            $table->integer('departmentID')->nullable(true);
            $table->integer('itemMatrixID')->nullable(true);
            $table->integer('manufacturerID')->nullable(true);
            $table->integer('seasonID')->nullable(true);
            $table->integer('defaultVendorID')->nullable(true);
            $table->timestamps();

            $table->primary('itemID');
        });

        Schema::create('category', function(Blueprint $table) {
            $table->integer('categoryID');
            $table->text('name')->nullable(true);
            $table->string('nodeDepth')->nullable(true);
            $table->text('fullPathName')->nullable(true);
            $table->integer('leftNode')->nullable(true);
            $table->integer('rightNode')->nullable(true);
            $table->timestamp('createTime')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->integer('parentID')->nullable(true);
            $table->timestamps();
            $table->primary('categoryID');
        });

        Schema::create('item_attribute_set', function (Blueprint $table) {
            $table->integer('itemAttributeSetID');
            $table->text('name')->nullable(true);
            $table->text('attributeName1')->nullable(true);
            $table->text('attributeName2')->nullable(true);
            $table->text('attributeName3')->nullable(true);

            $table->primary('itemAttributeSetID');
            $table->timestamps();
        });


        Schema::create('item_shop', function (Blueprint $table) {
            $table->integer('itemShopID');
            $table->integer('qoh')->nullable(true);
            $table->integer('backorder')->nullable(true);
            $table->integer('componentQoh')->nullable(true);
            $table->integer('componentBackorder')->nullable(true);
            $table->integer('reorderPoint')->nullable(true);
            $table->integer('sellable')->nullable(true);
            $table->integer('reorderLevel')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->integer('shopID')->nullable(true);

            $table->primary('itemShopID');
            $table->timestamps();
        });

        Schema::create('item_component', function (Blueprint $table) {
            $table->integer('itemComponentID');
            $table->integer('quantity')->nullable(true);
            $table->integer('componentGroup')->nullable(true);
            $table->integer('assemblyItemID')->nullable(true);
            $table->integer('componentItemID')->nullable(true);

            $table->primary('itemComponentID');
            $table->timestamps();
        });

        Schema::create('item_price', function (Blueprint $table) {
            $table->integer('itemID');
            $table->float('amount')->nullable(true);
            $table->string('useType')->nullable(true);
            $table->integer('useTypeID')->nullable(true);

            $table->primary(['itemID', 'useType']);
            $table->timestamps();
        });

        Schema::create('item_vendor_num', function (Blueprint $table) {
            $table->integer('itemVendorNumID');
            $table->string('value')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->integer('vendorID')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('itemVendorNumID');
            $table->timestamps();
        });

//        Schema::create('custom_field_values', function (Blueprint $table) {
//
//        });

        Schema::create('serialized', function (Blueprint $table) {
            $table->integer('serializedID');
            $table->text('colorName')->nullable(true);
            $table->text('sizeName')->nullable(true);
            $table->text('serial')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->integer('saleLineID')->nullable(true);
            $table->integer('customerID')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('serializedID');
            $table->timestamps();
        });

        Schema::create('transfer', function (Blueprint $table) {
            $table->integer('transferID');
            $table->boolean('sent')->nullable(true);
            $table->boolean('received')->nullable(true);
            $table->integer('note')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->integer('transferFromID')->nullable(true);
            $table->integer('transferToID')->nullable(true);
            $table->integer('orderID')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('transferID');
            $table->timestamps();
        });

        Schema::create('transfer_item', function (Blueprint $table) {
            $table->integer('transferItemID');
            $table->integer('toSend')->nullable(true);
            $table->integer('toReceive')->nullable(true);
            $table->integer('sent')->nullable(true);
            $table->integer('received')->nullable(true);
            $table->float('sentValue')->nullable(true);
            $table->float('receivedValue')->nullable(true);
            $table->text('comment')->nullable(true);
            $table->integer('transferID')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('transferItemID');
            $table->timestamps();
        });

        Schema::create('inventory_transfer', function (Blueprint $table) {
            $table->integer('transferID');
            $table->text('note')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->timestamp('createTime')->nullable(true);
            $table->string('status')->nullable(true);
            $table->timestamp('sentOn')->nullable(true);
            $table->timestamp('needBy')->nullable(true);
            $table->integer('sendingShopID')->nullable(true);
            $table->integer('sentByEmployeeID')->nullable(true);
            $table->integer('receivingShopID')->nullable(true);

            $table->primary('transferID');
            $table->timestamps();
        });

        Schema::create('inventory_transfer_item', function (Blueprint $table) {
            $table->integer('transferItemID');
            $table->integer('toSend')->nullable(true);
            $table->integer('toReceive')->nullable(true);
            $table->integer('sent')->nullable(true);
            $table->integer('received')->nullable(true);
            $table->float('sentValue')->nullable(true);
            $table->float('receivedValue')->nullable(true);
            $table->text('comment')->nullable(true);
            $table->integer('transferID')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);

            $table->primary('transferItemID');
            $table->timestamps();
        });

        Schema::create('inventory_count', function (Blueprint $table) {
            $table->integer('inventoryCountID');
            $table->text('name')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->integer('shopID')->nullable(true);

            $table->primary('inventoryCountID');
            $table->timestamps();
        });

        Schema::create('inventory_count_calc', function (Blueprint $table) {
            $table->integer('inventoryCountCalcID');
            $table->timestamp('timeStamp')->nullable(true);
            $table->integer('calcQoh')->nullable(true);
            $table->integer('countedQoh')->nullable(true);
            $table->integer('inventoryCountID')->nullable(true);
            $table->integer('itemID')->nullable(true);

            $table->primary('inventoryCountCalcID');
            $table->timestamps();
        });

        Schema::create('inventory_count_item', function (Blueprint $table) {
            $table->integer('inventoryCountItemID');
            $table->integer('qty')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->integer('inventoryCountID')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->integer('employeeID')->nullable(true);

            $table->primary('inventoryCountItemID');
            $table->timestamps();
        });

        Schema::create('inventory_count_reconcile', function (Blueprint $table) {
            $table->integer('inventoryCountReconcileID');
            $table->timestamp('createTime')->nullable(true);
            $table->float('costChange')->nullable(true);
            $table->integer('qohChange')->nullable(true);
            $table->integer('inventoryCountID')->nullable(true);
            $table->integer('itemID')->nullable(true);

            $table->primary('inventoryCountReconcileID');
            $table->timestamps();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('inventory');
    }
}
