<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateSalesOrders extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('sale', function (Blueprint $table) {

            $table->integer('saleID');
            $table->timestamp('timeStamp')->nullable(true);
            $table->float('discountPercent')->nullable(true);
            $table->boolean('completed')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->boolean('voided')->nullable(true);
            $table->boolean('enablePromotions')->nullable(true);
            $table->timestamp('createTime')->nullable(true);
            $table->timestamp('updateTime')->nullable(true);
            $table->timestamp('completeTime')->nullable(true);
            $table->string('referenceNumber')->nullable(true);
            $table->string('referenceNumberSource')->nullable(true);
            $table->float('tax1Rate')->nullable(true);
            $table->float('tax2Rate')->nullable(true);
            $table->float('change')->nullable(true);
            $table->float('calcDiscount')->nullable(true);
            $table->float('calcTotal')->nullable(true);
            $table->float('calcSubtotal')->nullable(true);
            $table->float('calcTaxable')->nullable(true);
            $table->float('calcNonTaxable')->nullable(true);
            $table->float('calcAvgCost')->nullable(true);
            $table->float('calcFIFOCost')->nullable(true);
            $table->float('calcTax1')->nullable(true);
            $table->float('calcTax2')->nullable(true);
            $table->float('calcPayments')->nullable(true);
            $table->float('total')->nullable(true);
            $table->float('totalDue')->nullable(true);
            $table->float('displayableTotal')->nullable(true);
            $table->float('balance')->nullable(true);
            $table->integer('customerID')->nullable(true);
            $table->integer('discountID')->nullable(true);
            $table->integer('employeeID')->nullable(true);
            $table->integer('quoteID')->nullable(true);
            $table->integer('registerID')->nullable(true);
            $table->integer('shipToID')->nullable(true);
            $table->integer('shopID')->nullable(true);
            $table->integer('taxCategoryID')->nullable(true);
            $table->integer('noteID')->nullable(true);

            $table->primary('saleID');
            $table->timestamps();
        });

        Schema::create('sale_account', function(Blueprint $table) {
            $table->integer('saleAccountID');
            $table->integer('creditAccountID')->nullable(true);
            $table->integer('salePaymentID')->nullable(true);
            $table->integer('saleLineID')->nullable(true);

            $table->primary('saleAccountID');
            $table->timestamps();
        });

        Schema::create('sale_line', function (Blueprint $table) {
            $table->integer('saleLineID');
            $table->timestamp('createTime')->nullable(true);
            $table->timestamp('timeStamp')->nullable(true);
            $table->integer('unitQuantity')->nullable(true);
            $table->float('unitPrice')->nullable(true);
            $table->float('normalUnitPrice')->nullable(true);
            $table->float('discountAmount')->nullable(true);
            $table->float('discountPercent')->nullable(true);
            $table->float('avgCost')->nullable(true);
            $table->float('fifoCost')->nullable(true);
            $table->float('tax')->nullable(true);
            $table->float('tax1Rate')->nullable(true);
            $table->float('tax2Rate')->nullable(true);
            $table->boolean('isLayaway')->nullable(true);
            $table->boolean('isWorkorder')->nullable(true);
            $table->boolean('isSpecialOrder')->nullable(true);
            $table->string('sortorder')->nullable(true);
            $table->integer('onCompletedTrans')->nullable(true);
            $table->float('displayableSubtotal')->nullable(true);
            $table->float('displayableUnitPrice')->nullable(true);
            $table->float('calcLineDiscount')->nullable(true);
            $table->float('calcTransactionDiscount')->nullable(true);
            $table->float('calcTotal')->nullable(true);
            $table->float('calcSubtotal')->nullable(true);
            $table->float('calcTax1')->nullable(true);
            $table->float('calcTax2')->nullable(true);
            $table->integer('taxClassID')->nullable(true);
            $table->integer('customerID')->nullable(true);
            $table->integer('discountID')->nullable(true);
            $table->integer('employeeID')->nullable(true);
            $table->integer('itemID')->nullable(true);
            $table->integer('noteID')->nullable(true);
            $table->integer('parentSaleLineID')->nullable(true);
            $table->integer('priceRuleID')->nullable(true);
            $table->integer('shopID')->nullable(true);
            $table->integer('taxCategoryID')->nullable(true);
            $table->integer('saleID')->nullable(true);

            $table->primary('saleLineID');
            $table->timestamps();
        });

        Schema::create('sale_payment', function (Blueprint $table) {
            $table->integer('salePaymentID');
            $table->float('amount')->nullable(true);
            $table->timestamp('createTime')->nullable(true);
            $table->boolean('archived')->nullable(true);
            $table->string('remoteReference')->nullable(true);
            $table->string('paymentID')->nullable(true);
            $table->integer('saleID')->nullable(true);
            $table->integer('paymentTypeID')->nullable(true);
            $table->integer('ccChargeID')->nullable(true);
            $table->integer('refPaymentID')->nullable(true);
            $table->integer('registerID')->nullable(true);
            $table->integer('employeeID')->nullable(true);
            $table->integer('creditAccountID')->nullable(true);

            $table->primary('salePaymentID');
            $table->timestamps();
        });

        Schema::create('sale_void', function(Blueprint $table) {
            $table->integer('saleVoidID');
            $table->timestamp('createTime')->nullable(true)->nullable(true);
            $table->text('reason')->nullable(true);
            $table->integer('saleID')->nullable(true);
            $table->integer('employeeID')->nullable(true);
            $table->integer('shopID')->nullable(true);

            $table->primary('saleVoidID');
            $table->timestamps();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('sales_orders');
    }
}
